@@ -218,11 +218,17 @@ public:
    * Notes on clip points:
       * - If input chip source is mutiband this will set all band to same
    *   percentage of clip.
+   * 
+   *  Note:  The midpoint is normalized between 0 to 1 where the default is 0.5 or the
+   *         average of the high low values when used.
    */
    void setMidPoint(const ossim_float64& value);
 
    /**
     * Sets the mid clip point for band.
+    * 
+    *  Note:  The midpoint is normalized between 0 to 1 where the default is 0.5 or the
+    *         average of the high low values when used.
     */
    void setMidPoint(const ossim_float64& value, ossim_uint32 zero_based_band);
 
@@ -438,6 +444,15 @@ public:
     */
    ossimFilename getHistogramFile() const;
    
+   void setBiasFactor(ossim_float64 factor);
+   ossim_float64 getBiasFactor()const;
+
+   void setAutoMinMaxBias(ossim_float64 bias);
+   ossim_float64 getAutoMinMaxBias()const;
+
+   void setPercentPenetration(ossim_float64 percent);
+   ossim_float64 getPercentPenetration()const;
+   
    /**
     * Method to the save the state to a keyword list.
     * Returns true if ok or false on error.
@@ -455,7 +470,7 @@ public:
                           const char* prefix=0);
    
    
-   virtual ostream& print(ostream& os) const;
+   virtual std::ostream& print(std::ostream& os) const;
 
    /**
     * Returns the min pixel of the band.
@@ -553,19 +568,31 @@ private:
    StretchMode                   theStretchMode;
    bool                          theDirtyFlag;
    mutable ossimRefPtr<ossimMultiResLevelHistogram>  theHistogram;
-   vector<ossim_float64>         theNormalizedLowClipPoint;
-   vector<ossim_float64>         theNormalizedHighClipPoint;
-   vector<ossim_float64>         theMidPoint;
-   vector<ossim_float64>         theMinOutputValue;
-   vector<ossim_float64>         theMaxOutputValue;
+   std::vector<ossim_float64>         theNormalizedLowClipPoint;
+   std::vector<ossim_float64>         theNormalizedHighClipPoint;
+   std::vector<ossim_float64>         theMidPoint;
+   std::vector<ossim_float64>         theMinOutputValue;
+   std::vector<ossim_float64>         theMaxOutputValue;
 
    // Maps zero based band to histogram band.
-   vector<ossim_uint32>          theBandList;
+   std::vector<ossim_uint32>          theBandList;
 
    // Internally bypassed flag.
    bool theBypassFlag;
    bool theResetBandIndicesFlag;
 
+   ossim_float64 m_percentPenetration;
+
+   // Internal bias used to find the clip points
+   ossim_float64 m_autoMinMaxBias;
+
+   // this is a multiplier for any of the current bias factors
+   // This will serve as a fudge factor for the Internal bias values.
+   //
+   // Will vary between 0 and 1
+   //
+   ossim_float64 m_biasFactor;
+
    TYPE_DATA
 };
 
