@@ -26,6 +26,8 @@
 #include <cstdlib>
 #include <iomanip>
 
+using namespace std;
+
 RTTI_DEF1(ossimHistogramRemapper, "ossimHistogramRemapper", ossimTableRemapper)
 
 static ossimTrace traceDebug("ossimHistogramRemapper:debug");
@@ -45,6 +47,9 @@ static const char MIN_OUTPUT_VALUE_KW[] = "min_output_value";
 static const char MAX_OUTPUT_VALUE_KW[] = "max_output_value";
 static const char STRETCH_MODE_KW[] = "stretch_mode";
 static const char HISTOGRAM_FILENAME_KW[] = "histogram_filename";
+static const char AUTO_MINMAX_BIAS_KW[] = "auto_minmax_bias";
+static const char BIAS_FACTOR_KW[] = "bias_factor";
+static const char PERCENT_PENETRATION_KW[] = "percent_penetration";
 
 #ifdef OSSIM_ID_ENABLED
 static const char OSSIM_ID[] = "$Id: ossimHistogramRemapper.cpp 23182 2015-03-09 14:30:52Z okramer $";
@@ -62,7 +67,10 @@ ossimHistogramRemapper::ossimHistogramRemapper()
    theMaxOutputValue(),
    theBandList(),
    theBypassFlag(true),
-   theResetBandIndicesFlag(false)
+   theResetBandIndicesFlag(false),
+   m_percentPenetration(0.05),
+   m_autoMinMaxBias(.006),
+   m_biasFactor(1.0)
 {
    setNumberOfInputs(2);
    if (traceDebug())
@@ -800,7 +808,24 @@ bool ossimHistogramRemapper::loadState(const ossimKeywordlist& kwl,
             return false;
          }
       }
-     
+      lookup = kwl.find(prefix, AUTO_MINMAX_BIAS_KW);
+      if(lookup)
+      {
+         m_autoMinMaxBias = ossimString(lookup).toFloat64();
+      }
+
+      lookup = kwl.find(prefix, BIAS_FACTOR_KW);
+      if(lookup)
+      {
+         m_biasFactor = ossimString(lookup).toFloat64();
+      }
+
+      lookup = kwl.find(prefix, PERCENT_PENETRATION_KW);
+      if(lookup)
+      {
+         m_percentPenetration = ossimString(lookup).toFloat64();
+      }
+           
       //---
       // Get the band specific keywords.
       // NOTES:
@@ -981,12 +1006,20 @@ bool ossimHistogramRemapper::saveState(ossimKeywordlist& kwl,
            true);
    
    const ossim_uint32 BANDS = (ossim_uint32)theNormalizedLowClipPoint.size();
-   
    kwl.add(prefix,
            ossimKeywordNames::NUMBER_BANDS_KW,
            BANDS,
            true);
-   
+
+
+ static const char AUTO_MINMAX_BIAS_KW[] = "auto_minmax_bias";
+static const char AUTO_MINMAX_BIAS_FACTOR_KW[] = "auto_minmax_bias_factor";
+static const char PERCENT_PENETRATION_KW[] = "percent_penetration";
+  
+   kwl.add(prefix, AUTO_MINMAX_BIAS_KW, m_autoMinMaxBias, true);
+   kwl.add(prefix, BIAS_FACTOR_KW, m_biasFactor, true);
+   kwl.add(prefix, PERCENT_PENETRATION_KW, m_percentPenetration, true);
+
    for (ossim_uint32 band = 0; band < BANDS; ++band)
    {
       ossimString keyword;
@@ -1397,27 +1430,33 @@ template <class T> void ossimHistogramRemapper::buildLinearTable(T /* dummy */)
       
       const T NULL_PIX = static_cast<T>(getNullPixelValue(band));
       const T MIN_PIX  = static_cast<T>(theMinOutputValue[band]);
-      const T MAX_PIX  = static_cast<T>(theMaxOutputValue[band]);
-      
+      const T MAX_PIX = static_cast<T>(theMaxOutputValue[band]);
+      const ossim_float64 TARGET_CENTER_PIX = static_cast<T>((MAX_PIX - MIN_PIX) * 0.5);
       ossim_float64 min_clip_value = h->LowClipVal(theNormalizedLowClipPoint[band]);
       ossim_float64 max_clip_value = h->HighClipVal(1.0-theNormalizedHighClipPoint[band]);
+      ossim_float64 centerClipValue = min_clip_value * (1.0 - theMidPoint[band]) + (max_clip_value * theMidPoint[band]);
       ossim_float64 gain = 1.0;
+      ossim_float64 gainLeft  = 1.0;
+      ossim_float64 gainRight = 1.0;
 
-      if(theTableType == ossimTableRemapper::NATIVE)
+      if (theTableType == ossimTableRemapper::NATIVE)
       {
          min_clip_value = floor(min_clip_value);
          max_clip_value = ceil(max_clip_value);
-         gain = (MAX_PIX-MIN_PIX+1)/(max_clip_value-min_clip_value+1);
+         gain = (MAX_PIX-MIN_PIX+1)/(max_clip_value-min_clip_value);
+         gainLeft = (TARGET_CENTER_PIX - MIN_PIX + 1) / (centerClipValue - min_clip_value);
+         gainRight = (MAX_PIX - TARGET_CENTER_PIX + 1) / (max_clip_value - centerClipValue);
       }
       else
       {
          gain = (MAX_PIX-MIN_PIX)/(max_clip_value-min_clip_value);
+         gainLeft = (TARGET_CENTER_PIX - MIN_PIX) / (centerClipValue - min_clip_value);
+         gainRight = (MAX_PIX - TARGET_CENTER_PIX) / (max_clip_value - centerClipValue);
       }
 
-
       table[index] = NULL_PIX;
       ++index;
-      
+
 #if 0 /* Please leave for debug. */
       std::cout  << "table count:    " << theTableBinCount
                  << "\nmin_clip_value: " << min_clip_value
@@ -1428,6 +1467,7 @@ template <class T> void ossimHistogramRemapper::buildLinearTable(T /* dummy */)
 #endif
 
       T pix = MIN_PIX;
+      bool needSkewCheck = std::fabs(theMidPoint[band] - 0.5) > FLT_EPSILON;
       for (ossim_uint32 pixIndex = 1; pixIndex < theTableBinCount; ++pixIndex)
       {
          ossim_float64 p = pix;
@@ -1441,13 +1481,24 @@ template <class T> void ossimHistogramRemapper::buildLinearTable(T /* dummy */)
          }
          else
          {
-            p = ossim::round<ossim_float64>( ((p - min_clip_value) * gain) + MIN_PIX );
-
-            // Range check...
-            p = ( p >= MIN_PIX ) ? ((p <= MAX_PIX) ? p : MAX_PIX ) : MIN_PIX;
-            
+            if (!needSkewCheck)
+            {
+               p = (((p - min_clip_value) * gain) + MIN_PIX);
+            }
+            else if (p < centerClipValue)
+            {
+               p = (((p - min_clip_value) * gainLeft) + MIN_PIX);
+            }
+            else if (p > centerClipValue)
+            {
+               p = (((p - centerClipValue) * gainRight) + TARGET_CENTER_PIX);
+            }
+            else
+            {
+               p = TARGET_CENTER_PIX;
+            }
+            p = (p >= MIN_PIX) ? ((p <= MAX_PIX) ? p : MAX_PIX) : MIN_PIX;
          }
-
          table[index] = static_cast<T>(p);
 
          // cout << "table[" << index << "]: " << table[index] << endl;
@@ -1477,18 +1528,17 @@ template <class T> void ossimHistogramRemapper::buildAutoLinearMinMaxTableTempla
       }
       return;
    }
+   ossim_float64 bias = m_autoMinMaxBias*m_biasFactor;
    for (ossim_uint32 band = 0; band < BANDS; ++band)
    {
       ossimRefPtr<ossimHistogram> h  = getHistogram(band);
       T nullPix = static_cast<T>(getNullPixelValue(band));
-      //h->fillInteriorEmptyBins();
 
-      if(h.valid())
+      if(h)
       {
          ossim_uint32 n     = h->GetRes();
          ossim_float64 low  = h->GetIndex(h->GetMinVal());
          ossim_float64 high = h->GetIndex(h->GetMaxVal());
-         double bias = 0.006;
          if(n > 0)
          {
             double newCount       = 0.0;
@@ -1496,7 +1546,7 @@ template <class T> void ossimHistogramRemapper::buildAutoLinearMinMaxTableTempla
             double percentage = 0.0;
             int idx = 0;
             const ossim_int64* counts = h->GetCounts();
-            double count   = h->ComputeArea();
+            double count              = h->ComputeArea();
             for(idx = 0; idx < (ossim_int32)(n-1); ++idx)
             {
                bool hasValue = (counts[idx] > 0)&&(counts[idx+1]);
@@ -1506,8 +1556,6 @@ template <class T> void ossimHistogramRemapper::buildAutoLinearMinMaxTableTempla
                }
                percentage = newCount / count;
                nextPercentage = (newCount + counts[idx+1]) / count;
-//               if (std::fabs(percentage - 0.006) <
-//                   std::fabs(nextPercentage - 0.006))
                if (hasValue &&((std::fabs(percentage - bias) <
                                 std::fabs(nextPercentage - bias))))
                {
@@ -1578,7 +1626,6 @@ void ossimHistogramRemapper::buildLinearTableStdFromMean()
    {
       multiplier =  3.0;
    }
-   
    // Finally, build the table...
    for (ossim_uint32 band = 0; band < BANDS; ++band)
    {
@@ -1645,8 +1692,8 @@ template <class T> void ossimHistogramRemapper::buildAutoLinearPercentileTableTe
       if(h.valid())
       {
          ossim_uint32 n     = h->GetRes();
-         ossim_float64 low  = h->GetIndex(h->LowClipVal((float)0.05));
-         ossim_float64 high = h->GetIndex(h->HighClipVal((float)0.05));
+         ossim_float64 low  = h->GetIndex(h->LowClipVal((float)m_percentPenetration));
+         ossim_float64 high = h->GetIndex(h->HighClipVal((float)m_percentPenetration));
          if(low > high)
          {
             low = 0;
@@ -1693,7 +1740,7 @@ void ossimHistogramRemapper::initializeClips(ossim_uint32 bands)
       {
          theNormalizedLowClipPoint[band]  = 0.0;
          theNormalizedHighClipPoint[band] = 1.0;
-         theMidPoint[band] = 0.0;
+         theMidPoint[band] = 0.5;
 
          switch(theOutputScalarType)
          {
@@ -2157,7 +2204,35 @@ double ossimHistogramRemapper::getMinPixelValue(ossim_uint32 band)const
 
    return result;
 }
+void ossimHistogramRemapper::setBiasFactor(ossim_float64 factor)
+{
+   m_biasFactor = factor;  
+}
+ossim_float64 ossimHistogramRemapper::getBiasFactor()const
+{
+   return m_biasFactor;
+}
 
+void ossimHistogramRemapper::setAutoMinMaxBias(ossim_float64 bias)
+{
+   m_autoMinMaxBias = bias;
+}
+
+ossim_float64 ossimHistogramRemapper::getAutoMinMaxBias()const
+{
+   return m_autoMinMaxBias;
+}
+
+void ossimHistogramRemapper::setPercentPenetration(ossim_float64 percent)
+{
+   m_percentPenetration = percent;
+}
+
+ossim_float64 ossimHistogramRemapper::getPercentPenetration()const
+{
+   return m_percentPenetration;
+}
+  
 double ossimHistogramRemapper::getMaxPixelValue(ossim_uint32 band)const
 {
    double result = ossimTableRemapper::getMaxPixelValue(band);
